@@ -1,6 +1,6 @@
 /*
  call table redirection for IA-64
- Copyright (c) 2006 Hitachi,Ltd.,
+ Copyright (c) 2006,2007 Hitachi,Ltd.,
  Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
@@ -127,13 +127,13 @@ struct ks_ia64_symbol *ia64_get_true_syscall(void)
 	return &true_sys_call_table[1][r15];
 }
 
-void pre_sys_call(const ctr_handler_t args)
+void pre_sys_call(const ctr_arg_t args)
 {
 	if (pre_handler)
 		pre_handler(args);
 }
 
-void post_sys_call(const ctr_handler_t args)
+void post_sys_call(const ctr_arg_t args)
 {
 	if (post_handler)
 		post_handler(args);
@@ -151,6 +151,7 @@ int change_sys_call_table(call_table_t *s_addr, call_table_t *fs_addr,
 	
 	true_sys_call_addr[0] = fs_addr;
 	true_sys_call_addr[1] = s_addr;
+
        	for (i = 0; i < NR_syscalls; i++) {
 		for (j = 0; j < KS_NR_TABLE; j++) {
 			if (j == 0) {
@@ -178,7 +179,7 @@ int change_sys_call_table(call_table_t *s_addr, call_table_t *fs_addr,
 				(call_table_t)((unsigned long)(*(void **)stb_addr_2) | 1UL);
 			s_addr[i] = 			
 				(call_table_t)((unsigned long)(*(void **)stb_addr_2) | 1UL);
-		}	
+		}
 	}
 	return 0;
 }
@@ -224,17 +225,21 @@ void unregister_ctr(void)
 /*------------------------------------------------------------------------
  *  module init/exit
  */
-#define CTR_INIT static int __init
-#define CTR_EXIT static void
-
-CTR_EXIT ia64_exit_ctr(void)
+static void ia64_exit_ctr(void)
 {
 	return;
 }
 
-CTR_INIT ia64_init_ctr(void)
+static int __init ia64_init_ctr(void)
 {
+	/* check parameter */
+	if ((void *)ks_gp == NULL) {
+		printk(KERN_ERR "error(init):Invalid parameter\n");
+		return -EINVAL;
+	}
+
 	init_kstrax_true_return_addr();
+
 	return 0;
 }
 
