@@ -1,7 +1,7 @@
 /*
  call table redirection for IA-64
- Copyright (c) 2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2006,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #ifndef __CTREDIRECT_H__
 #define __CTREDIRECT_H__
 
-typedef struct ctr_args {
+typedef struct ctr_arg_list {
 	long arg_1;
 	long arg_2;
 	long arg_3;
@@ -31,18 +31,18 @@ typedef struct ctr_args {
 	long arg_7;
 	long arg_8;
 	long arg_9;
-} ctr_handler_t;
+} ctr_arg_t;
 
-typedef long asmlinkage (*call_table_t) (ctr_handler_t);
-typedef void (*syscall_handler_t) (const ctr_handler_t);
+typedef long asmlinkage (*call_table_t) (ctr_arg_t);
+typedef void (*syscall_handler_t) (const ctr_arg_t);
 
 asmlinkage int kstrax_set_return_addr(unsigned long, unsigned long, unsigned long,
 				      unsigned long);
 asmlinkage struct ks_ia64_asmdata *kstrax_get_return_addr(void);
 unsigned long kstrax_get_syscall_num(void);
 struct ks_ia64_symbol *ia64_get_true_syscall(void);
-void pre_sys_call(const ctr_handler_t);
-void post_sys_call(const ctr_handler_t);
+void pre_sys_call(const ctr_arg_t);
+void post_sys_call(const ctr_arg_t);
 int change_sys_call_table(call_table_t *, call_table_t *, 
 			  void **, void **);
 void restore_sys_call_table(void);
