@@ -1,8 +1,8 @@
 /*
  stub for IA-64
- Copyright (c) 2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
- Updated by Masami Hiramatsu <hiramatu@sdl.hitachi.co.jp>
+ Copyright (c) 2006,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
+ Updated by Masami Hiramatsu <masami.hiramatsu.pt@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -38,28 +38,6 @@ MODULE_PARM_DESC(fs_addr, "Address of system call table(sys_call_table).");
 extern void sys_call_stub_arch(void);
 extern void sys_call_stub_arch_clone(void);
 
-asmlinkage long sys_call_stub(ctr_handler_t args)
-{
-        long ret = -1;
-        struct ks_ia64_symbol *true_syscall;
-
-	true_syscall = ia64_get_true_syscall();
-
-        /*
-         * pre processing
-         */
-	pre_sys_call(args);
-
-        ret = ((call_table_t)true_syscall)(args);
-        /*
-         * post processing
-         */
-	args.arg_6 = ret;
-	post_sys_call(args);
-	
-        return ret;
-}
-
 /*------------------------------------------------------------------------
  *  module init/exit
  */
@@ -92,7 +70,5 @@ static int __init init_stub(void)
 
 module_init(init_stub);
 module_exit(exit_stub);
-MODULE_AUTHOR("S.Moriya <s-moriya@sdl.hitachi.co.jp>");
+MODULE_AUTHOR("S.Moriya <satoru.moriya.br@hitachi.com>");
 MODULE_LICENSE("GPL");
-EXPORT_SYMBOL_GPL(sys_call_stub);
-
