@@ -1,7 +1,7 @@
 /*
  kstrax ioctl header
- Copyright (c) 2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2006,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <linux/unistd.h>
 
 #define NR_TRACE_PROCESS_MAX 256  
+#define NR_TRACE_SYSCALL_MAX 300
 
 #define TRACE_ALL      -1
 #define TRACE_FILE     -2
@@ -42,26 +43,20 @@
 #define MAGIC_SYSCALL_ZERO 23
 
 #ifdef _IA64_KSTRAX_
-# ifndef NR_syscalls
-# define NR_syscalls 256
-# endif
 
 #define TRACE_TYPE_LIMIT 8
 
 typedef struct kstrax_status {
 	int nr_cpu;
+	int nr_syscalls;
 	int flag_spec_syscall;
-	int trace_syscall[NR_syscalls];
+	int trace_syscall[NR_TRACE_SYSCALL_MAX];
 	pid_t trace_pid[NR_TRACE_PROCESS_MAX];
 	int nr_buf_entry;
 } trace_stat_t;
 
 #else /* _IA64_KSTRAX_ */
 
-# ifndef NR_syscalls
-# define NR_syscalls 285
-# endif
-
 #define TRACE_SOCKET   -8
 #define TRACE_BIND     -9
 #define TRACE_CONNECT -10
@@ -100,8 +95,9 @@ typedef struct kstrax_status {
 
 typedef struct kstrax_status {
 	int nr_cpu;
+	int nr_syscalls;
 	int flag_spec_syscall;
-	int trace_syscall[NR_syscalls];
+	int trace_syscall[NR_TRACE_SYSCALL_MAX];
 	int trace_socketcall[NR_SOCKETCALL];
 	int trace_ipc[NR_IPC];
 	pid_t trace_pid[NR_TRACE_PROCESS_MAX];
