@@ -1,7 +1,7 @@
 /*
  mbuffer -- buffer module
- Copyright (c) 2005,2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2005,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -81,7 +81,7 @@ int init_mbuffer(int entry)
 		head->size = entry;
 		head->w_index = 0;
 		head->r_index = 0;
-		head->cur_index = -1;
+		head->limit_index = -1;
 		head->wakeup_index = -1;
 		head->wakeup_flag = KSTRAX_WAKEUP;
 		init_waitqueue_head(&head->wq_head);
@@ -109,6 +109,7 @@ int kstrax_init_status(int nr_entry)
 	int i;
 
 	kstrax_status.nr_cpu = num_online_cpus();
+	kstrax_status.nr_syscalls = NR_syscalls;
 	kstrax_status.flag_spec_syscall = 0;
 	for (i = 0; i < NR_TRACE_PROCESS_MAX; i++)
 		kstrax_status.trace_pid[i] = INIT_PID;
@@ -122,7 +123,7 @@ int kstrax_init_status(int nr_entry)
 	return 0;
 }
 
-void pre_sys_call_record(const ctr_handler_t args)
+void pre_sys_call_record(const ctr_arg_t args)
 {
 	int i;
 	buf_head_t *head;
@@ -141,8 +142,10 @@ void pre_sys_call_record(const ctr_handler_t args)
 		}
 	}
 
-	if(__check_tracing_syscall(&args, &kstrax_status) == NOT_TRACE)
-		return;
+	if (kstrax_status.flag_spec_syscall != 0) {
+		if(__check_tracing_syscall(&args, &kstrax_status) == NOT_TRACE)
+			return;
+	}
 
 	/* buffer header of current cpu */
 	cpu = get_cpu();
@@ -171,7 +174,7 @@ void pre_sys_call_record(const ctr_handler_t args)
 	put_cpu();
 }
 
-void post_sys_call_record(const ctr_handler_t args)
+void post_sys_call_record(const ctr_arg_t args)
 {
 	int i;
 	buf_head_t *head;
@@ -182,7 +185,7 @@ void post_sys_call_record(const ctr_handler_t args)
 	/* check tracing pid or not */
 	if (kstrax_status.trace_pid[0] != INIT_PID) {
 		for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
-			if (current->pid == kstrax_status.trace_pid[i]) {
+			if (kstrax_status.trace_pid[i] == current->pid) {
 				break;
 			} else if (kstrax_status.trace_pid[i] == INIT_PID) {
 				return ;
@@ -190,8 +193,10 @@ void post_sys_call_record(const ctr_handler_t args)
 		}
 	}
 
-	if(__check_tracing_syscall(&args, &kstrax_status) == NOT_TRACE)
-		return;;
+	if (kstrax_status.flag_spec_syscall != 0) {
+		if(__check_tracing_syscall(&args, &kstrax_status) == NOT_TRACE)
+			return;
+	}
 
 	/* buffer header of current cpu */
 	cpu = get_cpu();
@@ -225,9 +230,7 @@ void post_sys_call_record(const ctr_handler_t args)
 /*--------------------------------------------------------------------------------
  *  called by ioctl function
  */
-static int register_trace_pid(pid_t);
-static int unregister_trace_pid(pid_t);
-static int unregister_trace_pid_all(void); 
+static int unregister_trace_pid(pid_t, int);
 static void kstrax_trace_all(void);
 static void kstrax_trace_file(void);
 static void kstrax_trace_process(void);
@@ -236,12 +239,14 @@ static void kstrax_trace_demo(void);
 
 int kstrax_copy_status(trace_stat_t *status)
 {
+	int retval = 0;
+
 	if (copy_to_user((trace_stat_t *)status, &kstrax_status, 
 			 sizeof(trace_stat_t))) {
 	      	printk(KERN_ERR "error(ioctl) : cannot copy date\n");
-		return -EINVAL;
+		retval = -EFAULT;
 	}
-	return 0;
+	return retval;
 }
 
 /* reset read index */
@@ -252,7 +257,6 @@ int kstrax_reset_index(void)
 	for_each_online_cpu(cpu) {
 		buf_head_t *head;
 		head = &per_cpu(kernel_buffer_head, cpu);
-
 		head->r_index = head->w_index;
 	}
 	return 0;
@@ -261,77 +265,66 @@ int kstrax_reset_index(void)
 /* user_buf is not pointer to int but sys_call_t */
 int kstrax_read_basetime(int *buf)
 {
+	int retval = 0;
 	sys_call_t tm_info;
 	__set_basetime(&tm_info);
-	return (copy_to_user((sys_call_t *)buf, &tm_info, sizeof(sys_call_t)));
+	if (copy_to_user((sys_call_t *)buf, &tm_info, sizeof(sys_call_t))) {
+		printk(KERN_ERR "error(ioctl) : cannot copy basetime");
+		retval = -EFAULT;
+	}
+	return retval;
 }
 
 int kstrax_pid_spec(pid_t pid)
 {
 	int i;
+	int retval = 0;
 
-	if (pid < -1)
-		return -EINVAL;
-
-	if (pid == -1) {
-		return unregister_trace_pid_all();
-	}
-
-	for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
-		if (pid == kstrax_status.trace_pid[i])
-			return unregister_trace_pid(pid);
-	}
-
-	return register_trace_pid(pid);
-}
-
-static int register_trace_pid(pid_t pid)
-{
-	int i;
-
-	for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
-		if (kstrax_status.trace_pid[i] == INIT_PID) {
-			kstrax_status.trace_pid[i] = pid;
-			break;
-		}
-	}
-
-	if (i == NR_TRACE_PROCESS_MAX) {
-		printk("error:cannot register pid(too much pid registered)\n");
-		return -1;
-	}
-	return 0;
-}
-
-static int unregister_trace_pid(pid_t pid)
-{
-	int i, j;
-	
-	for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
-		if (kstrax_status.trace_pid[i] != pid)
-			continue;
-
-		for (j = i+1; j < NR_TRACE_PROCESS_MAX; j++) {
-			if (kstrax_status.trace_pid[j] == INIT_PID)
-				break;
-		}
-	     
-		if (j == i+1) {
+	if (pid < -1) {
+		retval = -EINVAL;
+	} else if (pid == -1) {
+		/* delete all pid from trace table --> from now trace all pid */
+		for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
 			kstrax_status.trace_pid[i] = INIT_PID;
-		} else {
-			kstrax_status.trace_pid[i] = kstrax_status.trace_pid[j-1];
-			kstrax_status.trace_pid[j-1] = INIT_PID;
+		}
+	} else {
+		/* 0 <= pid */
+		for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
+			if (kstrax_status.trace_pid[i] == pid) {
+				/* delete pid from trace table */
+				retval = unregister_trace_pid(pid, i);
+				break;
+			} else if (kstrax_status.trace_pid[i] == INIT_PID) {
+				/* add pid to trace table */
+				kstrax_status.trace_pid[i] = pid;
+				break;
+			}
+		}
+		if (i == NR_TRACE_PROCESS_MAX) {
+			printk("error:cannot register pid"
+			       "(too much pid registered)\n");
+			retval = -EINVAL;
 		}
 	}
-	return 0;
+	return retval;
 }
 
-static int unregister_trace_pid_all(void) 
+static int unregister_trace_pid(pid_t pid, int index)
 {
-	int i;
-	for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
-		kstrax_status.trace_pid[i] = INIT_PID;
+	int j;
+	
+	for (j = index + 1; j < NR_TRACE_PROCESS_MAX; j++) {
+		if (kstrax_status.trace_pid[j] == INIT_PID)
+			break;
 	}
+	
+	if (j == index + 1) {
+		kstrax_status.trace_pid[index] = INIT_PID;
+	} else {
+		kstrax_status.trace_pid[index] = kstrax_status.trace_pid[j-1];
+		kstrax_status.trace_pid[j-1] = INIT_PID;
+	}
+	
 	return 0;
 }
 
@@ -385,107 +378,102 @@ static void kstrax_trace_all(void)
 static void kstrax_trace_file(void)
 {
 	int i;
-	
-	/* off specified syscall */
-	if (kstrax_status.flag_spec_syscall == 1 &&
-	    SPECIFIED_FILE == 1) {
+	if (kstrax_status.flag_spec_syscall == 1 && SPECIFIED_FILE == 1) {
+		/* stop tracing FILE system calls */
 		for (i = 0; i < NR_syscalls; i++) {
 			IS_FILE_SYSCALL(i) 
 				kstrax_status.trace_syscall[i] = 0;
 		}
 		test_flag_spec_syscall();	
-		return;
+	} else {
+		/* start tracing FILE system calls */
+		for (i = 0; i < NR_syscalls; i++) {
+			IS_FILE_SYSCALL(i)
+				kstrax_status.trace_syscall[i] = 1;
+		}
+		kstrax_status.flag_spec_syscall = 1;
 	}
-	
-	for (i = 0; i < NR_syscalls; i++) {
-		IS_FILE_SYSCALL(i)
-			kstrax_status.trace_syscall[i] = 1;
-	}
-	kstrax_status.flag_spec_syscall = 1;
 }
 
 static void kstrax_trace_process(void)
 {
 	int i;
-
-	/* off specified syscall */
 	if (kstrax_status.flag_spec_syscall == 1 &&
 	    SPECIFIED_PROCESS == 1) {
+		/* stop tracing PROCESS management system calls */
 		for (i = 0; i < NR_syscalls; i++) {
 			IS_PROCESS_SYSCALL(i) 
 				kstrax_status.trace_syscall[i] = 0;
 		}
 		test_flag_spec_syscall();	
-		return;
+	} else {
+		/* start tracing PROCESS management system calls */
+		for (i = 0; i < NR_syscalls; i++) {
+			IS_PROCESS_SYSCALL(i)
+				kstrax_status.trace_syscall[i] = 1;
+		}
+		kstrax_status.flag_spec_syscall = 1;
 	}
-
-	for (i = 0; i < NR_syscalls; i++) {
-		IS_PROCESS_SYSCALL(i)
-			kstrax_status.trace_syscall[i] = 1;
-	}
-	kstrax_status.flag_spec_syscall = 1;
 }
 
 static void kstrax_trace_signal(void)
 {
 	int i;
-
-	/* off specified syscall */
 	if (kstrax_status.flag_spec_syscall == 1 &&
 	    SPECIFIED_SIGNAL == 1) {
+		/* stop tracing SIGNAL system calls */
 		for (i = 0; i < NR_syscalls; i++) {
 			IS_SIGNAL_SYSCALL(i) 
 				kstrax_status.trace_syscall[i] = 0;
 		}
 		test_flag_spec_syscall();	
-		return;
+	} else {
+		/* start tracing SIGNAL system calls */
+		for (i = 0; i < NR_syscalls; i++) {
+			IS_SIGNAL_SYSCALL(i)
+				kstrax_status.trace_syscall[i] = 1;
+		}
+		kstrax_status.flag_spec_syscall = 1;
 	}
-
-	for (i = 0; i < NR_syscalls; i++) {
-		IS_SIGNAL_SYSCALL(i)
-			kstrax_status.trace_syscall[i] = 1;
-	}
-
-	kstrax_status.flag_spec_syscall = 1;
 }
 
 static void kstrax_trace_demo(void)
 {
 	int i;
-	
-	/* off specified syscall */
 	if (kstrax_status.flag_spec_syscall == 1 &&
 	    SPECIFIED_DEMO == 1) {
+		/* stop tracing system calls for demo */
 		for (i = 0; i < NR_syscalls; i++) {
 			IS_DEMO_SYSCALL(i)
 				kstrax_status.trace_syscall[i] = 0;
 		}
 		test_flag_spec_syscall();
-		return;
+	} else {
+		/* start tracing system calls for demo */
+		for (i = 0; i < NR_syscalls; i++) {
+			IS_DEMO_SYSCALL(i)
+				kstrax_status.trace_syscall[i] = 1;
+		}
+		kstrax_status.flag_spec_syscall = 1;
 	}
-
-	for (i = 0; i < NR_syscalls; i++) {
-		IS_DEMO_SYSCALL(i)
-			kstrax_status.trace_syscall[i] = 1;
-	}
-	kstrax_status.flag_spec_syscall = 1;
 }
 
 static int kstrax_trace_each_syscall(int syscall_num)
 {	
+	int retval = 0;
+
 	if (syscall_num > NR_syscalls) {
 		printk("error(syscall_spec):bad argument\n");
-		return -EINVAL;
+		retval = -EINVAL;
+	} else {
+		if (kstrax_status.trace_syscall[syscall_num] == 0) {
+			kstrax_status.trace_syscall[syscall_num] = 1;
+		} else {
+			kstrax_status.trace_syscall[syscall_num] = 0;
+		}
+		test_flag_spec_syscall();
 	}
-	
-	if (kstrax_status.trace_syscall[syscall_num] == 0)
-		kstrax_status.trace_syscall[syscall_num] = 1;
-	else
-		kstrax_status.trace_syscall[syscall_num] = 0;
-	
-	test_flag_spec_syscall();
-	
-	return 0;
+	return retval;
 }
 
 static void test_flag_spec_syscall(void)
@@ -507,7 +495,7 @@ int kstrax_set_current_index(void)
 
 	for_each_online_cpu(cpu) {
 		head = &per_cpu(kernel_buffer_head, cpu);
-		head->cur_index = head->w_index;
+		head->limit_index = head->w_index;
 	}
 	return 0;
 }
@@ -518,9 +506,8 @@ int kstrax_set_current_index(void)
 ssize_t kstrax_read(struct file *filp, char __user *buf, 
 		     size_t count, loff_t *f_pos)
 {
-	int request;
+	int request_nr_entry, copied_nr_entry;
 	int start = 0, end = 0;
-	int csize;
 	int copy_ret;
 	__u32 cpu;  /* unsigned int */
 	buf_head_t *head;
@@ -536,7 +523,7 @@ ssize_t kstrax_read(struct file *filp, char __user *buf,
 	head = &per_cpu(kernel_buffer_head, cpu);
 
 	/* translate byte into index */
-	request = count / sizeof(sys_call_t);
+	request_nr_entry = count / sizeof(sys_call_t);
 
  retry:
 	start = head->r_index;
@@ -545,42 +532,44 @@ ssize_t kstrax_read(struct file *filp, char __user *buf,
 	if (start > end)
 		end = head->size;
 
-	csize = end - start;
-	if (request < csize) {
-		end = (start + request) % head->size;
-		csize = end - start;
+	copied_nr_entry = end - start;
+	if (request_nr_entry < copied_nr_entry) {
+		end = (start + request_nr_entry) % head->size;
+		copied_nr_entry = end - start;
 	}
 
-	if (start < head->cur_index && head->cur_index < end) {
-		end = head->cur_index;
-		csize = end - start;
+	if (start < head->limit_index && head->limit_index < end) {
+		end = head->limit_index;
+		copied_nr_entry = end - start;
 	}
 
 	/* copy */
 	copy_ret = copy_to_user(buf, (head->buffer + start), 
-				sizeof(sys_call_t) * csize);
+				sizeof(sys_call_t) * copied_nr_entry);
 	if (copy_ret) {
 		printk("Error at copy_to_user, cpu:%d\n", smp_processor_id());
 		goto out;
 	}
 
-	request -= csize;
+	request_nr_entry -= copied_nr_entry;
 	head->r_index = end % head->size;
-	buf += csize * sizeof(sys_call_t);
-	*f_pos += csize * sizeof(sys_call_t);
+	buf += copied_nr_entry * sizeof(sys_call_t);
+	*f_pos += copied_nr_entry * sizeof(sys_call_t);
 
-	if (head->cur_index != -1 && head->cur_index == end)
+	/* if kstrax command is executed with -R option --> limit_index != -1
+	 * This option read kernel buffer only one round.
+	 */
+	if (head->limit_index != -1 && head->limit_index == end)
 		goto out;
 
-
 	/* if need one more copy */
-	if (0 < request) {
+	if (0 < request_nr_entry) {
 		int r_to_w, r_to_up;
 		
 		/* calculate distance from r_index to wake up */
 		r_to_up = head->size / 2;
-		if (r_to_up > request)
-			r_to_up = request;
+		if (r_to_up > request_nr_entry)
+			r_to_up = request_nr_entry;
 		
 		if (head->w_index >= head->r_index) {
 			r_to_w = head->w_index - head->r_index;
@@ -588,12 +577,7 @@ ssize_t kstrax_read(struct file *filp, char __user *buf,
 			r_to_w = head->size - head->r_index;
 			r_to_w += head->w_index;
 		}
-#if 0
-		if ((filp->f_flags & O_NONBLOCK) &&
-		    head->cur_index != -1) {
-			goto out;
-		}
-#endif		
+
 		/* sleep */
 		if (r_to_w < r_to_up) {
 			/* termination */
@@ -606,11 +590,12 @@ ssize_t kstrax_read(struct file *filp, char __user *buf,
 			head->wakeup_index = (head->r_index + r_to_up) % head->size;
 			head->wakeup_flag = KSTRAX_SLEEP;
 
-			wait_event_interruptible(head->wq_head, head->wakeup_flag == KSTRAX_WAKEUP); 
+			wait_event_interruptible(head->wq_head, 
+						 head->wakeup_flag == KSTRAX_WAKEUP); 
 		}
 		goto retry;
 	}
 	
  out:
-	return (count - ((ssize_t)request * sizeof(sys_call_t)));
+	return (count - ((ssize_t)request_nr_entry * sizeof(sys_call_t)));
 }
