@@ -1,7 +1,7 @@
 /*
  mbuffer -- buffer module
- Copyright (c) 2005,2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2005,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ int kstrax_syscall_spec(int);
 int kstrax_set_current_index(void);
 ssize_t kstrax_read(struct file *, char __user *, size_t, loff_t *);
 
-void pre_sys_call_record(const ctr_handler_t);
-void post_sys_call_record(const ctr_handler_t);
+void pre_sys_call_record(const ctr_arg_t);
+void post_sys_call_record(const ctr_arg_t);
 
 #endif /* __MBUFFER_H__ */
