@@ -1,7 +1,7 @@
 /*
  mcontrol -- control module(event handler)
- Copyright (c) 2005,2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2005,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include "ctr.h"
 
 #define DEVICE_NAME		"kstrax"
-#define VERSION                 "1.0.0"
+#define VERSION                 "1.0.2"
 #define	DEV_COUNT		1
 #define KERNEL_BUFFER_ENTRY	4000 * 40  /* multiples of 128 */
 
@@ -143,7 +143,7 @@ static int kstrax_ioctl(struct inode *inode, struct file *flip,
 		retval = kstrax_set_current_index_ioctl();
 		break;
 	default:
-		retval = -1;
+		retval = -EINVAL;
 	}
 	return retval;
 
@@ -157,15 +157,19 @@ static int kstrax_status_ioctl(trace_stat_t *user_status)
 static int kstrax_set_cpu_ioctl(int *user_cpu)
 {
 	int cpu;
+	int retval = 0;
+	
 	if (copy_from_user(&cpu, user_cpu, sizeof(int))) {
 		printk(KERN_ERR "error(ioctl):cannot copy cpu infomation\n");
-		return -EINVAL;
+		retval = -EFAULT;
+		goto end;
 	}
 
 	if (set_cpus_allowed(current, cpumask_of_cpu(cpu)) < 0) {
-		return -EINVAL;
+		retval = -EINVAL;
 	}
-	return 0;
+ end:
+	return retval;
 }
 
 static int kstrax_init_index_ioctl(void)
@@ -182,15 +186,20 @@ static int kstrax_read_basetime_ioctl(int *user_buf)
 
 static int kstrax_set_owner_ioctl(pid_t *user_pid)
 {
+	int retval = 0;
+	
 	if (copy_from_user(&kstrax_owner, user_pid, sizeof(pid_t))) {
 		printk(KERN_ERR "error(ioctl):cannot copy owner information\n");
-		return -EINVAL;
+		retval = -EFAULT;
+		goto end;
 	}
+
 	if (kstrax_owner < 0) {
 		kstrax_owner = -1;
-		return -EINVAL;
+		retval = -EINVAL;
 	}
-	return 0;
+ end:
+	return retval;
 }
 
 static int kstrax_clr_owner_ioctl(void)
@@ -204,7 +213,7 @@ static int kstrax_pid_spec_ioctl(pid_t *user_pid)
 	pid_t pid;
 	if (copy_from_user(&pid, user_pid, sizeof(pid_t))) {
 		printk(KERN_ERR "error(ioctl):cannot copy owner information\n");
-		return -EINVAL;
+		return -EFAULT;
 	}
 	return kstrax_pid_spec(pid);
 }
@@ -213,8 +222,9 @@ static int kstrax_syscall_spec_ioctl(int *user_syscall_num)
 {
 	int syscall_num;
 	if (copy_from_user(&syscall_num, user_syscall_num, sizeof(int))) {
-		printk(KERN_ERR "error(ioctl):cannot copy syscall spec information\n");
-		return -EINVAL;
+		printk(KERN_ERR "error(ioctl):"
+		       "cannot copy syscall spec information\n");
+		return -EFAULT;
 	}
 	return kstrax_syscall_spec(syscall_num);
 }
@@ -227,10 +237,6 @@ static int kstrax_set_current_index_ioctl(void)
 /*-------------------------------------------------------------------------------------
  *  module init/exit
  */
-
-MODULE_AUTHOR("S.Moriya <s-moriya@sdl.hitachi.co.jp>");
-MODULE_LICENSE("GPL");
-
 static void exit_kstrax(void)
 {
 	/* release  /dev/kstrax */
@@ -242,7 +248,11 @@ static void exit_kstrax(void)
 	/* unregister save functions */
 	unregister_ctr();
 
+#ifndef USE_SYNC_RCU
 	synchronize_kernel();
+#else
+	synchronize_rcu();
+#endif /* USE_SYNC_RCU */
 
 	clear_mbuffer();	
 }
@@ -271,6 +281,12 @@ static int __init init_kstrax(void)
 		goto fail;
 	}
 
+	retval = kstrax_init_status(kbuffer_entry);
+	if (retval < 0) {
+		printk(KERN_ERR "error(init):cannot initialize status\n");
+		goto fail;
+	}
+
 	/* determine the module's major and minor number */
 	retval = alloc_chrdev_region(&kstrax_number, 0, DEV_COUNT, DEVICE_NAME);
 	if (retval < 0) {
@@ -285,11 +301,6 @@ static int __init init_kstrax(void)
 		goto fail;
 	}
 
-	retval = kstrax_init_status(kbuffer_entry);
-	if (retval < 0) {
-		printk(KERN_ERR "error(init):cannot initialize status\n");
-		goto fail;
-	}
 	printk("%s module version %s\n", DEVICE_NAME, VERSION);		
 	return retval; // success
 	
@@ -300,3 +311,5 @@ static int __init init_kstrax(void)
 
 module_init(init_kstrax);
 module_exit(exit_kstrax);
+MODULE_AUTHOR("S.Moriya <satoru.moriya.br@hitachi.com>");
+MODULE_LICENSE("GPL");
