@@ -1,7 +1,7 @@
 /*
  syscall_name - system call information for RedHat Enterprise Linux AS 4
- Copyright (c) 2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2006,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ struct syscall_attribute syscall_name[] = {
 	[1] = {"exit", 1, 0},
 	[2] = {"read", 3, 2},
 	[3] = {"write", 3, 2},
-	[4] = {"open", 2, 1},
+	[4] = {"open", 3, 1},
 	[5] = {"close", 1, 0},
 	[6] = {"creat", 2, 1},
 	[7] = {"link", 2, 3},
@@ -172,7 +172,7 @@ struct syscall_attribute syscall_name[] = {
 	[148] = {"mmap2", 6, 0},
 	[149] = {"pciconfig_read", 5, 16},
 	[150] = {"pciconfig_write", 5, 16},
-	[151] = {"perfmonctl", 9, 4},
+	[151] = {"perfmonctl", 4, 4},
 	[152] = {"sigaltstack", 9, 3},
 	[153] = {"rt_sigaction", 4, 6},
 	[154] = {"rt_sigpending", 2, 1},
@@ -276,5 +276,30 @@ struct syscall_attribute syscall_name[] = {
 	[252] = {"set_zone_reclaim", 3, 0},
 	[253] = {"inotify_init", 0, 0},
 	[254] = {"inotify_add_watch", 3, 2},
-	[255] = {"inotify_rm_watch", 2, 0}
+	[255] = {"inotify_rm_watch", 2, 0},
+	[256] = {"migrate_pages", 4, 12},
+	[257] = {"openat", 4, 2},
+	[258] = {"mkdirat", 3, 2},
+	[259] = {"mknodat", 4, 2},
+	[260] = {"fchownat", 5, 2},
+	[261] = {"futimesat", 3, 6},
+	[262] = {"newstatat", 4, 6},
+	[263] = {"unlinkat", 3, 2},
+	[264] = {"renameat", 4, 10},
+	[265] = {"linkat", 5, 10},
+	[266] = {"symlinkat", 3, 5},
+	[267] = {"readlinkat", 4, 6},
+	[268] = {"fchmodat", 3, 2},
+	[269] = {"faccessat", 3, 2},
+	[270] = {"pselect", 6, 62},
+	[271] = {"ppoll", 5, 15},
+	[272] = {"unshare", 1, 0},
+	[273] = {"splice", 6, 10},
+	[274] = {"set_robust_list", 2, 1},
+	[275] = {"get_robust_list", 3, 6},
+	[276] = {"sync_file_range", 4, 0},
+	[277] = {"tee", 3, 0},
+	[278] = {"vmsplice", 4, 2},
+	[279] = {"move_pages", 6, 28},
+	[280] = {"getcpu", 3, 7}
 };
