@@ -1,7 +1,7 @@
 /*
  kstrax
- Copyright (c) 2005,2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2005,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 #include "kstrax_arch.h"
 
 #define DEFAULT_NR_ENTRY 4000
-#define DEFAULT_LIMIT_TIME 0 
+#define DEFAULT_LIMIT_TIME 0 /* sec */
 #define NAME       "kstrax"
 #define COPYRIGHT  "COPYRIGHT (C) HITACHI,LTD. 2005,2006"
 #define INPUT_FILE "/dev/kstrax"
@@ -47,23 +47,23 @@
  */
 extern char *optarg;
 int kstrax_cflag = 0, kstrax_kflag = 0, kstrax_rflag = 0;
-int kstrax_Rflag = 0, kstrax_dflag = 0, kstrax_gflag = 0; 
+int kstrax_Rflag = 0, kstrax_gflag = 0; 
 static int kstrax_bflag = 0, kstrax_eflag = 0, kstrax_oflag = 0;
 static int kstrax_pflag = 0, kstrax_sflag = 0;
 static int kstrax_tflag = 0, kstrax_Tflag = 0;
-
+static int ks_nr_syscalls = 0;
 /*
  *  function declaration
  */
 static void kstrax_set_output_filename(char *, const char *);
 static void kstrax_set_default_bin_filename(char *, const char *);
 static void kstrax_set_default_txt_filename(const char *, char *);
-static void kstrax_set_trace_syscall(char *);
-static void kstrax_set_trace_pid(char *);
+static void kstrax_set_trace_syscall(const char *);
+static void kstrax_set_trace_pid(const char *);
 static void kstrax_usage(void);
 static void kstrax_version(void);
 static void kstrax_get_status(struct kstrax_status *);
-static void kstrax_print_status(struct kstrax_status *);
+static void kstrax_print_status(const struct kstrax_status *);
 static int kstrax_strtol(const char *);
 static void kstrax_restore_status(struct kstrax_status *, struct kstrax_status *);
 static void kstrax_set_snapshot(void);
@@ -72,13 +72,15 @@ int main(int argc, char *argv[])
 {
 	
 	char c;
-	char input_file[1024] = INPUT_FILE, output_file[1024] = "\0";
+	char input_file[KSTRAX_FNAME_MAX] = INPUT_FILE;
+	char output_file[KSTRAX_FNAME_MAX] = "\0";
 	int nr_entry = DEFAULT_NR_ENTRY, limit_time = DEFAULT_LIMIT_TIME; /* sec */
 	struct kstrax_status old_status = {0}, new_status={0};
 
-	while ((c = getopt(argc, argv, "+bd:c:e:ghko:p:r:Rs:St:T:v")) != EOF) {
+	while ((c = getopt(argc, argv, "+bc:e:ghko:p:r:Rs:St:T:v")) != EOF) {
 		switch (c) {
 		case 'b': /* output binary */
+			kstrax_get_status(&new_status);
 			kstrax_bflag = 1;
 			break;
 		case 'c': /* count calls, times... print statistics information */
@@ -89,20 +91,9 @@ int main(int argc, char *argv[])
 			strncpy(input_file, optarg, (KSTRAX_FNAME_MAX - 5));
 			kstrax_cflag = 1;
 			break;
-		/*------------------------ken-patu-------------------------*/
-		case 'd':
-			if (strlen(optarg) > (KSTRAX_FNAME_MAX - 5)) {
-				printf("Too long filename\n");
-				exit(1);
-			}
-			strncpy(input_file, optarg, (KSTRAX_FNAME_MAX - 5));
-			kstrax_dflag = 1;
-			break;
-		/*---------------------------------------------------------*/
 		case 'e': /* system call specification in kernel space */
-			if (new_status.nr_cpu != 0)
-				new_status.nr_cpu = 0;
-			kstrax_get_status(&old_status);
+			if (old_status.nr_cpu == 0)
+				kstrax_get_status(&old_status);
 			kstrax_set_trace_syscall(optarg);
 			kstrax_get_status(&new_status);
 			kstrax_print_status(&new_status);
@@ -122,9 +113,8 @@ int main(int argc, char *argv[])
 			kstrax_oflag = 1;
 			break;
 		case 'p': /* pid specification in kernel space */
-			if (new_status.nr_cpu != 0)
-				new_status.nr_cpu = 0;
-			kstrax_get_status(&old_status);
+			if (old_status.nr_cpu == 0)
+				kstrax_get_status(&old_status);
 			kstrax_set_trace_pid(optarg);
 			kstrax_get_status(&new_status);
 			kstrax_print_status(&new_status);
@@ -187,10 +177,9 @@ int main(int argc, char *argv[])
 	/* exclusion check */
 	if (((kstrax_bflag || kstrax_kflag || kstrax_sflag || kstrax_Tflag ||
 	      kstrax_Rflag) &&
-	     (kstrax_cflag || kstrax_rflag || kstrax_tflag || kstrax_dflag || 
-	      kstrax_gflag)) ||
+	     (kstrax_cflag || kstrax_rflag || kstrax_tflag || kstrax_gflag)) ||
 	    (kstrax_cflag && kstrax_rflag) || (kstrax_rflag && kstrax_tflag) ||
-	    (kstrax_tflag && kstrax_cflag)) {
+	    (kstrax_tflag && kstrax_cflag) || (kstrax_rflag && kstrax_gflag)) {
 		fprintf(stderr, "option confliction\n");
 		exit(1);
 	}
@@ -201,8 +190,7 @@ int main(int argc, char *argv[])
 		kstrax_read(nr_entry, limit_time, input_file, output_file);
 		if (kstrax_eflag == 1 || kstrax_pflag == 1)
 			kstrax_restore_status(&old_status, &new_status);
-	} else if (kstrax_cflag == 1 || kstrax_rflag == 1 || 
-		   kstrax_tflag == 1 || kstrax_dflag == 1) {
+	} else if (kstrax_cflag == 1 || kstrax_rflag == 1 || kstrax_tflag == 1) {
 		if (kstrax_oflag == 0)
 			kstrax_set_default_txt_filename(input_file, output_file);
 		kstrax_print(input_file, output_file);
@@ -213,7 +201,7 @@ int main(int argc, char *argv[])
 	return 0;
 }
 
-static void kstrax_set_trace_pid(char *optarg)
+static void kstrax_set_trace_pid(const char *optarg)
 {
 	int fd;
 	pid_t pid;
@@ -224,13 +212,13 @@ static void kstrax_set_trace_pid(char *optarg)
 		exit(1);
 	}
 	if (ioctl(fd, KSTRAX_IOC_PID_SPEC, &pid) < 0) {
-		perror("kstrax_set_trace_pic(ioctl)");
+		perror("kstrax_set_trace_pid(ioctl)");
 		exit(1);		
 	}
 	close(fd);
 }
 
-static void kstrax_set_trace_syscall(char *optarg)
+static void kstrax_set_trace_syscall(const char *optarg)
 {
 
 	int i, fd, ioctl_arg;
@@ -238,11 +226,6 @@ static void kstrax_set_trace_syscall(char *optarg)
 	char type[TRACE_TYPE_LIMIT][8] = {"ALL", "FILE", "NETWORK", "IPC",
 					  "PROCESS", "SIGNAL", "DEMO"};
 
-	if ((fd = open(INPUT_FILE, O_WRONLY)) < 0) {
-		perror("kstrax_set_trace_syscall(open)");
-		exit(1);
-	}
-
 	/* specified by syscall type */
 	for (i = 0; i < TRACE_TYPE_LIMIT; i++) {
 		if (strcmp(optarg, type[i]) == 0) {
@@ -253,11 +236,11 @@ static void kstrax_set_trace_syscall(char *optarg)
 
 	/* specified by syscall name */
 	if (flag_specified == 0) {
-		for (i = 0; i < NR_syscalls; i++) {
-			if (strcmp(optarg, syscall_name[i].name) != 0)
-				continue;
-			flag_specified = 1;
-			ioctl_arg = i;
+		for (i = 0; i < ks_nr_syscalls; i++) {
+			if (strcmp(optarg, syscall_name[i].name) == 0) {
+				ioctl_arg = i;
+				flag_specified = 1;
+			}
 		}
 	}
 
@@ -267,22 +250,22 @@ static void kstrax_set_trace_syscall(char *optarg)
 		fprintf(stderr, "Unknown System Call : %s\n", optarg);
 		exit(1);
 	}
+
+	if ((fd = open(INPUT_FILE, O_WRONLY)) < 0) {
+		perror("kstrax_set_trace_syscall(open)");
+		exit(1);
+	}
 	if (ioctl(fd, KSTRAX_IOC_SYSCALL_SPEC, &ioctl_arg) < 0) {
 		perror("kstrax_set_trace_syscall(ioctl)");
 		exit(1);
 	}
-
 	close(fd);
-
 }
 
 static void kstrax_get_status(struct kstrax_status *status)
 {
 	int fd;
 
-	if (status->nr_cpu != 0)
-		return;
-
 	if ((fd = open(INPUT_FILE, O_WRONLY)) < 0) {
 		perror("kstrax_get_status(open)");
 		exit(1);
@@ -291,10 +274,11 @@ static void kstrax_get_status(struct kstrax_status *status)
 		perror("kstrax_get_status(ioctl)");
 		exit(1);		
 	}
+	kstrax_set_nr_syscalls(status->nr_syscalls);
 	close(fd);
 }
 
-static void kstrax_print_status(struct kstrax_status *status)
+static void kstrax_print_status(const struct kstrax_status *status)
 {
 	int i;
 
@@ -321,54 +305,49 @@ static void kstrax_print_status(struct kstrax_status *status)
 		__kstrax_status_print_syscall_name(status);
 }
 
+static void kstrax_restore_trace_pid_half(struct kstrax_status *former,
+					  struct kstrax_status *latter,
+					  int fd)
+{
+	int i, j;
+	int flag_no_change;
+
+	for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
+		flag_no_change = 0;
+		if (former->trace_pid[i] == INIT_PID)
+			break;
+		for (j = 0; j < NR_TRACE_PROCESS_MAX; j++) {
+			if (latter->trace_pid[j] == INIT_PID)
+				break;
+			if (former->trace_pid[i] == latter->trace_pid[j]) {
+				flag_no_change = 1;
+			}
+		}
+		if (flag_no_change == 0) {
+			if (ioctl(fd, KSTRAX_IOC_PID_SPEC, 
+				  &former->trace_pid[i]) < 0) {
+				perror("kstrax_restore_trace_pid_half(ioctl)");
+				exit(1);
+			}
+		}
+	}
+}
+
 static void kstrax_restore_status(struct kstrax_status *old, 
 				  struct kstrax_status *new)
 {
-	int i, j;
 	int fd;
 
 	if ((fd = open(INPUT_FILE, O_WRONLY)) < 0) {
-		perror("kstrax_get_status(open)");
+		perror("kstrax_restore_status(open)");
 		exit(1);
-	}
-	
-	__kstrax_restore_trace_syscall(old, new, fd);
+	}	
 
-	for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
-		int flag_no_change = 0;
-		if (old->trace_pid[i] == INIT_PID)
-			break;
-		for (j = 0; j < NR_TRACE_PROCESS_MAX; j++) {
-			if (old->trace_pid[i] == new->trace_pid[j])
-				flag_no_change = 1;
-		}
-		if (flag_no_change == 0) {
-			if (ioctl(fd, KSTRAX_IOC_PID_SPEC, 
-				  &old->trace_pid[i]) < 0) {
-				perror("kstrax_set_trace_pic(ioctl)");
-				exit(1);		
-			}
-		}
-	}
-	for (i = 0; i < NR_TRACE_PROCESS_MAX; i++) {
-		int flag_no_change = 0;
-		if (new->trace_pid[i] == INIT_PID)
-			break;
-		for (j = 0; j < NR_TRACE_PROCESS_MAX; j++) {
-			if (new->trace_pid[i] == old->trace_pid[j])
-				flag_no_change = 1;
-		}
-		if (flag_no_change == 0) {
-			if (ioctl(fd, KSTRAX_IOC_PID_SPEC, 
-				  &new->trace_pid[i]) < 0) {
-				perror("kstrax_set_trace_pic(ioctl)");
-				exit(1);		
-			}
-		}
-	}
+	__kstrax_restore_trace_syscall(old, new, fd);
+	kstrax_restore_trace_pid_half(old, new, fd);
+	kstrax_restore_trace_pid_half(new, old, fd);
 
 	close(fd);
-
 }
 
 
@@ -379,7 +358,7 @@ static void kstrax_set_default_bin_filename(char *filename, const char *optarg)
 	struct tm *my_time;
 	
 	gethostname(host, HOST_NAME_MAX);
-	if (strlen(host) > (KSTRAX_FNAME_MAX - 17)) {
+	if (strlen(host) > (KSTRAX_FNAME_MAX - 13)) {
 		printf("Too long filename\n");
 		exit(1);
 	}
@@ -397,7 +376,7 @@ static void kstrax_set_default_txt_filename(const char *ifile, char *ofile)
 {
 	if (kstrax_rflag == 1)
 		sprintf(ofile, "%s_raw", ifile);
-	else if(kstrax_cflag == 1)
+	else if (kstrax_cflag == 1)
 		sprintf(ofile, "%s_stat", ifile);
 	else
 		sprintf(ofile, "%s_txt", ifile);
@@ -418,23 +397,23 @@ static void kstrax_usage(void)
 	printf("\nThis controls status of KSTRAX.\n");
 	printf("Usage:\n\t%s [option(s)]\n\n", NAME);
 	printf("<OPTION>\n");
-	printf("-b \t\t\tOutput buffer data\n");
+	printf("-b \t\t\tRetrieve trace data from kernel buffer.\n");
 	printf("-c [filename]\t\tPrint statistics information"
 	       " from binary file[name]\n");
-	printf("-e [syscall type]\tSet tracing syscall type"
-	       "[all/file/network/ipc/process/signal]\n");
+	printf("-e [syscall(type)]\tSet tracing syscall or syscall type"
+	       "[ALL/FILE/NETWORK/IPC/PROCESS/SIGNAL]\n");
 	printf("-h\t\t\tPrint this message.\n");
 	printf("-k\t\t\tNot clear kernel buffer\n");
 	printf("-o [filename]\t\tSet output file name\n");
 	printf("-p [pid]\t\tSet tracing pid[pid]\n");
 	printf("-r [filename]\t\tPrint system call information on console"
 	       " from binary file[name] at RAW mode\n");
-	printf("-R \t\t\tOutput buffer data (snap shot)");
+	printf("-R \t\t\tOutput buffer data (snap shot)\n");
 	printf("-s [number of entry]\tSet buffer size to [number of entry]\n");
 	printf("-S\t\t\tPrint trace status\n");
 	printf("-t [filename]\t\tPrint system call information"
-	       " from binary file[name] at text\n");
-	printf("-T [sec]\t\tPrint to binary file for [sec]seconds\n");
+	       " from binary file[name] to text\n");
+	printf("-T [sec]\t\tPrint binary file for [sec]seconds\n");
 	printf("-v\t\t\tPrint Version information\n\n");
 	exit(0);
 }
@@ -463,12 +442,23 @@ static void kstrax_set_snapshot(void)
 	int fd;
 
 	if ((fd = open(INPUT_FILE, O_WRONLY)) < 0) {
-		perror("kstrax_get_status(open)");
+		perror("kstrax_set_snapshot(open)");
 		exit(1);
 	}
 	if (ioctl(fd, KSTRAX_IOC_SET_CUR_INDEX) < 0) {
-		perror("kstrax_get_status(ioctl)");
+		perror("kstrax_set_snapshot(ioctl)");
 		exit(1);		
 	}
 	close(fd);
 }
+
+void kstrax_set_nr_syscalls(int num)
+{
+	if (ks_nr_syscalls == 0)
+		ks_nr_syscalls = num;
+}
+
+int kstrax_get_nr_syscalls(void)
+{
+	return ks_nr_syscalls;
+}
