@@ -1,7 +1,7 @@
 /*
  kstrax
- Copyright (c) 2005,2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2005,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -21,10 +21,13 @@
 #ifndef _KSTRAX_H_
 #define _KSTRAX_H_
 
-#define KSTRAX_VERSION    "1.0.0"
+#define KSTRAX_VERSION    "1.0.2"
 #define KSTRAX_FNAME_MAX  1024 
 
 extern int kstrax_cflag, kstrax_kflag, kstrax_rflag;
-extern int kstrax_Rflag, kstrax_dflag, kstrax_gflag;
+extern int kstrax_Rflag, kstrax_gflag;
+
+void kstrax_set_nr_syscalls(int);
+int kstrax_get_nr_syscalls(void);
 
 #endif  /* _KSTRAX_H_ */
