@@ -1,7 +1,7 @@
 /*
  kstrax_buffer
- Copyright (c) 2005,2006 Hitachi,Ltd.,
- Created by Satoru Moriya <s-moriya@sdl.hitachi.co.jp>
+ Copyright (c) 2005,2007 Hitachi,Ltd.,
+ Created by Satoru Moriya <satoru.moriya.br@hitachi.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -65,6 +65,7 @@ typedef struct kstrax_list_head {
 
 typedef struct kstrax_binary_header {
 	int id;
+	int nr_syscalls;
 	char version[10];
 	buffer_t buf_basetime;
 } bin_header_t;
@@ -72,10 +73,10 @@ typedef struct kstrax_binary_header {
 /*
  *  global variable
  */
-static int read_file_handle;
+static int module_file_handle;
 static tparam_t *kstrax_tparam;
-long kstrax_sum_proc_time[NR_syscalls] = {0};
-long kstrax_sum_count[NR_syscalls] = {0};
+proc_time_t *kstrax_sum_proc_time;
+syscall_count_t *kstrax_sum_count;
 
 #ifdef _IA64_KSTRAX_
 long kstrax_first_time = 0;
@@ -92,22 +93,22 @@ static void kstrax_sig_handler(int signum)
 	int flags;
 	struct kstrax_status sparam;
 
-	if (ioctl(read_file_handle, KSTRAX_IOC_SET_FORCE_WAKEUP)) {
+	if (ioctl(module_file_handle, KSTRAX_IOC_SET_FORCE_WAKEUP)) {
 		perror("kstrax_sig_handler(ioctl:force wakeup)");
 		exit(1);
 	}
-	flags = fcntl(read_file_handle, F_GETFL);
-	fcntl(read_file_handle, F_SETFL, flags|O_NONBLOCK);
+	flags = fcntl(module_file_handle, F_GETFL);
+	fcntl(module_file_handle, F_SETFL, flags|O_NONBLOCK);
 	
-	if (ioctl(read_file_handle, KSTRAX_IOC_STATUS, &sparam) < 0) {
+	if (ioctl(module_file_handle, KSTRAX_IOC_STATUS, &sparam) < 0) {
 		perror("kstrax_sig_handler(ioctl:get status)");
 		exit(1);
 	}
 	kstrax_for_each_online_cpu(cpu, sparam.nr_cpu) {
 		pthread_kill(kstrax_tparam[cpu].th, SIGUSR1);
 	}
-	
-	fprintf(stderr, "terminating...\n");
+
+	fprintf(stdout, "terminating...\n");
 	return;
 }
 
@@ -159,25 +160,25 @@ void kstrax_read(int nr_entry, int limit_time,
 	signal(SIGALRM, &kstrax_sig_handler);
 	signal(SIGUSR1, &kstrax_dummy_handler);
 
-	read_file_handle = kstrax_file_open(input_f, READ_FILE); 
+	module_file_handle = kstrax_file_open(input_f, READ_FILE); 
 
 	/* set owner process */
 	pid = getpid();
-	if (ioctl(read_file_handle, KSTRAX_IOC_SET_OWNER, &pid) < 0) {
+	if (ioctl(module_file_handle, KSTRAX_IOC_SET_OWNER, &pid) < 0) {
 		perror("kstrax_read(ioctl:set owner)");
 		exit(1);
 	}
 	
 	/* reset read index */
 	if (kstrax_kflag != 1) {
-		if (ioctl(read_file_handle, KSTRAX_IOC_INIT_INDEX) < 0) {
+		if (ioctl(module_file_handle, KSTRAX_IOC_INIT_INDEX) < 0) {
 			perror("kstrax_read(ioctl:init index)");
 			exit(1);
 		}
 	}
 	
  	/* read system status */
-	if (ioctl(read_file_handle, KSTRAX_IOC_STATUS, &sparam) < 0) {
+	if (ioctl(module_file_handle, KSTRAX_IOC_STATUS, &sparam) < 0) {
 		perror("kstrax_read(ioctl:read status)");
 		exit(1);
 	}
@@ -194,7 +195,7 @@ void kstrax_read(int nr_entry, int limit_time,
 		char s[1024];
 		sprintf(s, "%s_%02d", output_f, cpu);
 		kstrax_tparam[cpu].cpu_id    = cpu;
-		kstrax_tparam[cpu].input_fd  = read_file_handle;
+		kstrax_tparam[cpu].input_fd  = module_file_handle;
 		kstrax_tparam[cpu].output_fd = kstrax_file_open(s, WRITE_FILE);
 		kstrax_tparam[cpu].nr_entry  = nr_entry; 
 		kstrax_tparam[cpu].limit     = limit_time;
@@ -206,7 +207,7 @@ void kstrax_read(int nr_entry, int limit_time,
 
 	if (kstrax_Rflag == 0)
 		pause();
-	
+
 	/* wait threads */
 	kstrax_for_each_online_cpu(cpu, sparam.nr_cpu) {
 		pthread_join(kstrax_tparam[cpu].th, NULL);
@@ -214,12 +215,12 @@ void kstrax_read(int nr_entry, int limit_time,
 	}		
 
 	/* clear owner process */
-	if (ioctl(read_file_handle, KSTRAX_IOC_CLR_OWNER) < 0) {
+	if (ioctl(module_file_handle, KSTRAX_IOC_CLR_OWNER) < 0) {
 		perror("kstrax_read(ioctl:clear owner)");
 		exit(1);
 	}
 
-	close(read_file_handle);
+	close(module_file_handle);
 }
 
 /*
@@ -228,6 +229,7 @@ void kstrax_read(int nr_entry, int limit_time,
 static void read_syscall_per_cpu(tparam_t *tparam)
 {
 	sigset_t mask_set;
+
 	if (ioctl(tparam->input_fd, KSTRAX_IOC_SET_CPU, &tparam->cpu_id)) {
 		perror("read_syscall_per_cpu(ioctl:set cpu affinity)");
 		exit(1);
@@ -300,6 +302,7 @@ static void format_bin_file(int read_handle, int write_handle)
 	
 	/* init binary file header */
 	bin_head.id = BIN_HEAD_ID;
+	bin_head.nr_syscalls = kstrax_get_nr_syscalls();
 	strncpy(bin_head.version, KSTRAX_VERSION, 10);
 	if (ioctl(read_handle, KSTRAX_IOC_READ_BASETIME, 
 		  &bin_head.buf_basetime) < 0) {
@@ -326,6 +329,7 @@ void kstrax_print(const char *input_f, const char *output_f)
 {
 	int i;
 	int nr_file;
+
 	per_cpu_t *kst_data;
 	FILE *w_fp;
 	char filename[1024];
@@ -337,27 +341,22 @@ void kstrax_print(const char *input_f, const char *output_f)
 		perror("kstrax_print(malloc)");
 		exit(1);
 	}
+
 	for (i = 0; i < nr_file; i++) {
 		sprintf(filename, "%s_%02d", input_f, i);
 		kst_data[i].fd = kstrax_file_open(filename, READ_FILE);
 		
 		check_bin_header(&kst_data[i]);
 	}
-
-	if (strcmp(output_f, "\0") == 0) {
-		w_fp = stdout;
-	} else {
-		w_fp = fopen(output_f, "w");
-		if (w_fp == NULL) {
-			perror("kstrax_print(fopen)");
-			exit(1);
-		}
+	w_fp = fopen(output_f, "w");
+	if (w_fp == NULL) {
+		perror("kstrax_print(fopen)");
+		exit(1);
 	}
-
+	
 	marge_and_print(kst_data, nr_file, w_fp);
 
-	if (w_fp != stdout)
-		fclose(w_fp);
+	fclose(w_fp);
 	for (i = 0; i < nr_file; i++) {
 		close(kst_data[i].fd);
 	}
@@ -400,6 +399,7 @@ static void check_bin_header(per_cpu_t *kst_data)
 		fprintf(stderr, "error:invalid header\n");
 		exit(1);
 	}
+	kstrax_set_nr_syscalls(bin_head.nr_syscalls);
 
 	if (strcmp(bin_head.version, KSTRAX_VERSION)) {
 		fprintf(stderr, "Warning:Another version binary file\n");
@@ -432,6 +432,7 @@ static void marge_and_print(per_cpu_t *kst_data, int nr_file, FILE *w_fp)
 	int min = 0;
 	int option = 0;
 	int nr_round = 0;
+	int nr_syscalls = kstrax_get_nr_syscalls();
 	unsigned long first_serial, last_serial = 0;
 	syscall_count_t total_syscall = 0,  total_nr_lost = 0;
 	print_data_t p_data;
@@ -449,9 +450,19 @@ static void marge_and_print(per_cpu_t *kst_data, int nr_file, FILE *w_fp)
 		lost_syscall_raw,
 		lost_syscall_stats};
 
+	/* alloc counter */
+	kstrax_sum_proc_time = 
+		(proc_time_t *)malloc(nr_syscalls * sizeof(proc_time_t));
+	kstrax_sum_count =
+		(syscall_count_t *)malloc(nr_syscalls * sizeof(syscall_count_t));
+	for (i = 0; i < nr_syscalls; i++) {
+		kstrax_sum_proc_time[i] = 0;
+		kstrax_sum_count[i] = 0;
+	}
+
 	if (kstrax_rflag == 1) {
 		option = 1;
-	} else if (kstrax_cflag == 1 || kstrax_dflag == 1 /*ken-patu*/) {
+	} else if (kstrax_cflag == 1) {
 		option = 2;
 	}
 
@@ -465,7 +476,7 @@ static void marge_and_print(per_cpu_t *kst_data, int nr_file, FILE *w_fp)
 	/* record first serial */
 	min = earliest_entry(kst_data, nr_file);	
 	if (min == KS_FINISHED)
-		return;
+		goto out;
 	first_serial = kst_data[min].buf.serial;
 	last_serial = first_serial - 1;
 
@@ -511,29 +522,26 @@ static void marge_and_print(per_cpu_t *kst_data, int nr_file, FILE *w_fp)
 			nr_round++;
 	}
 
+	(*post_print[option])(&p_data);
+
 	if (nr_round) {
 		total_syscall = (syscall_count_t)((nr_round - 1) * ULONG_MAX + 
 					    (ULONG_MAX - first_serial) + last_serial);
 	} else {
 		total_syscall = (syscall_count_t)(last_serial - first_serial + 1);
 	}
-	(*post_print[option])(&p_data);
 
-	if (kstrax_gflag == 0)
+	if (kstrax_gflag == 0) {
 		__print_syscall_count(p_data.w_fp, total_syscall, total_nr_lost);
-#if 0		
-	fprintf(p_data.w_fp, 
-		"\n\ntotal syscalls      :: %lld\n"
-		"total lost syscalls :: %lld\n",
-		total_syscall / 2, total_nr_lost / 2);
-//#if 0 /* ken-patu */
-	if (kstrax_dflag == 0) {
-		fprintf(p_data.w_fp, 
-			"\n\ntotal syscalls      :: %lld\n"
-			"total lost syscalls :: %lld\n",
-			total_syscall / 2, total_nr_lost / 2);
+	} else {
+		if (kstrax_cflag == 0) {
+			__print_syscall_count_graph(p_data.w_fp);
+		}
 	}
-#endif 
+
+ out:
+	free(kstrax_sum_proc_time);
+	free(kstrax_sum_count);
 }
 
 static void read_entry(per_cpu_t *kst_data)
@@ -608,9 +616,11 @@ static statistics_t *alloc_statistics_array(int);
 /* "-t" */
 static void pre_print_normal(print_data_t *p_data)
 {
+	int nr_syscalls = kstrax_get_nr_syscalls();
+
 	p_data->free_list = NULL;
-	p_data->hash_table = alloc_hash_table(NR_syscalls);
-	p_data->stats = NULL;
+	p_data->hash_table = alloc_hash_table(nr_syscalls);
+	p_data->stats_array = NULL;
 
 	if (kstrax_gflag == 0) {
 		fprintf(p_data->w_fp, 
@@ -625,7 +635,7 @@ static void pre_print_normal(print_data_t *p_data)
 	} else  {
 		int i;
 		fprintf(p_data->w_fp, "name ");
-		for (i = 0; i < NR_syscalls; i++) {
+		for (i = 0; i < nr_syscalls; i++) {
 			fprintf(p_data->w_fp, "%s ", syscall_name[i].name);
 		}
 		fprintf(p_data->w_fp, "\n");
@@ -638,7 +648,7 @@ static void pre_print_raw(print_data_t *p_data)
 {
 	p_data->free_list = NULL;
 	p_data->hash_table = NULL;
-	p_data->stats = NULL;
+	p_data->stats_array = NULL;
 
 	fprintf(p_data->w_fp, 
 		"------------------------------------------------"
@@ -653,12 +663,12 @@ static void pre_print_raw(print_data_t *p_data)
 /* "-c" */
 static void pre_print_stats(print_data_t *p_data)
 {
-	int nr_stats;
-	nr_stats = __get_nr_stats();
+	int nr_stats_entry = __get_nr_syscalls();
+	int nr_syscalls = kstrax_get_nr_syscalls();
 
 	p_data->free_list = NULL;
-	p_data->hash_table = alloc_hash_table(NR_syscalls);
-	p_data->stats = alloc_statistics_array(nr_stats);
+	p_data->hash_table = alloc_hash_table(nr_syscalls);
+	p_data->stats_array = alloc_statistics_array(nr_stats_entry);
 
 	if (kstrax_gflag == 0) {
 		fprintf(p_data->w_fp, 
@@ -674,22 +684,6 @@ static void pre_print_stats(print_data_t *p_data)
 			"------------------------------------------------"
 			"-------------------------------\n");
 	}
-#if 0 /* ken-patu */
-	if (kstrax_dflag == 0) {
-		fprintf(p_data->w_fp, 
-			"----------------------------------------"
-			"----------------------------------------"
-			"-----------------------------------------------\n");
-		fprintf(p_data->w_fp, 
-			" ID  NAME                        count       "
-			" average            max            min       "
-			"   total   error  no entry  no return\n");
-		fprintf(p_data->w_fp, 
-			"----------------------------------------"
-			"----------------------------------------"
-			"-----------------------------------------------\n");
-	}
-#endif 
 }
 
 static kst_list_entry_t **alloc_hash_table(int nr_entry)
@@ -739,7 +733,7 @@ static void print_return_entry(const buffer_t *, FILE *);
 static void print_pair_entry(const buffer_t *, const buffer_t *, FILE *);
 static void print_call_entry_raw(FILE *, buffer_t *);
 static void print_return_entry_raw(FILE *, buffer_t *);
-static kst_list_entry_t *copy_entry(const buffer_t *, kst_list_entry_t **);
+static kst_list_entry_t *create_hash_entry(const buffer_t *, kst_list_entry_t **);
 static void add_pair_entry(const buffer_t *, const buffer_t *, statistics_t *);
 static void add_no_pair_entry(const buffer_t *, statistics_t *);
 static void list_free(kst_list_entry_t **);
@@ -750,12 +744,13 @@ static kst_list_entry_t *list_del(kst_list_entry_t **);
 static void main_print_normal(print_data_t *p_data)
 {
 	int hash;
+	int nr_syscalls = kstrax_get_nr_syscalls();
 	buffer_t *buffer;
 	kst_list_entry_t *copy, *del;
-	kst_list_entry_t **free_list_head, **hash_table;
+	kst_list_entry_t **flist_head, **hash_table;
 	
 	buffer = p_data->buf;
-	free_list_head = &p_data->free_list;
+	flist_head = &p_data->free_list;
 	hash_table = p_data->hash_table;
 
 	if (buffer->sys_call_number > 0) { /* call entry */
@@ -765,18 +760,17 @@ static void main_print_normal(print_data_t *p_data)
 				print_call_entry(buffer, p_data->w_fp);
 			return;
 		}	
-		copy = copy_entry(buffer, free_list_head);
-		hash = copy->buf.sys_call_number % NR_syscalls;
+		copy = create_hash_entry(buffer, flist_head);
+		hash = copy->buf.sys_call_number % nr_syscalls;
 		list_add(copy, &hash_table[hash]);
 	} else { /* return entry */
 		kst_list_entry_t **now;
 
-		hash = (-buffer->sys_call_number) % NR_syscalls;
-		now = &hash_table[hash];
-	
-		if (__print_sigreturn_normal(now, buffer, p_data) == PRINT_END)
+		hash = (-buffer->sys_call_number) % nr_syscalls;
+		if (__print_sigreturn_normal(hash, buffer, p_data) == PRINT_END)
 			return;
 
+		now = &hash_table[hash];			
 		while (*now) {
 			if ((*now)->buf.pid == buffer->pid) {
 				if (kstrax_gflag == 0)
@@ -787,7 +781,7 @@ static void main_print_normal(print_data_t *p_data)
 								 buffer,
 								 p_data->w_fp);
 				del = list_del(now);
-				list_add(del, free_list_head);
+				list_add(del, flist_head);
 				return;
 			} else {
 				now = &(*now)->next;
@@ -795,13 +789,13 @@ static void main_print_normal(print_data_t *p_data)
 		}
 		if (*now == NULL) {
 			if (buffer->sys_call_number == 0) {
-				copy = copy_entry(buffer, free_list_head);
+				copy = create_hash_entry(buffer, flist_head);
 				list_add(copy, &hash_table[hash]);
-				return;
+			} else {
+				/* there is not call entry but only return entry */
+				if (kstrax_gflag == 0)
+					print_return_entry(buffer, p_data->w_fp);
 			}
-			/* there is not call entry but only return entry */
-			if (kstrax_gflag == 0)
-				print_return_entry(buffer, p_data->w_fp);
 		}
 	}
 }
@@ -809,11 +803,12 @@ static void main_print_normal(print_data_t *p_data)
 /* "-r" */
 static void main_print_raw(print_data_t *p_data)
 {
+	int nr_syscalls = kstrax_get_nr_syscalls();
 	buffer_t *buffer;
 	
 	buffer = p_data->buf;
 	
-	if(NR_syscalls < abs(buffer->sys_call_number)) {
+	if(nr_syscalls < abs(buffer->sys_call_number)) {
 		fprintf(p_data->w_fp, "Unknown syscall number : %d\n", 
 			buffer->sys_call_number);
 		return;
@@ -840,15 +835,16 @@ static void main_print_raw(print_data_t *p_data)
 static void main_print_stats(print_data_t *p_data)
 {
 	int hash;
+	int nr_syscalls = kstrax_get_nr_syscalls();
 	buffer_t *buffer;
 	kst_list_entry_t *copy, *del;
-	kst_list_entry_t **free_list_head, **hash_table;
+	kst_list_entry_t **flist_head, **hash_table;
 	statistics_t *stats;
 
 	buffer = p_data->buf;
-	free_list_head = &p_data->free_list;
+	flist_head = &p_data->free_list;
 	hash_table = p_data->hash_table;
-	stats = p_data->stats;
+	stats = p_data->stats_array;
 
 	if (buffer->sys_call_number > 0) { /* call entry */
 		__IS_NO_PAIR {
@@ -856,23 +852,22 @@ static void main_print_stats(print_data_t *p_data)
 			add_no_pair_entry(buffer, stats);
 			return;
 		}	
-		copy = copy_entry(buffer, free_list_head);
-		hash = copy->buf.sys_call_number % NR_syscalls;
+		copy = create_hash_entry(buffer, flist_head);
+		hash = copy->buf.sys_call_number % nr_syscalls;
 		list_add(copy, &hash_table[hash]);
 	} else { /* return entry */
 		kst_list_entry_t **now;
 
-		hash = (-buffer->sys_call_number) % NR_syscalls;
-		now = &hash_table[hash];
-
-		if (__print_sigreturn_stats(now, buffer, p_data) == PRINT_END)
+		hash = (-buffer->sys_call_number) % nr_syscalls;
+		if (__print_sigreturn_stats(hash, buffer, p_data) == PRINT_END)
 			return;
 
+		now = &hash_table[hash];
 		while (*now) {
 			if ((*now)->buf.pid == buffer->pid) {
 				add_pair_entry(&(*now)->buf, buffer, stats);
 				del = list_del(now);
-				list_add(del, free_list_head);
+				list_add(del, flist_head);
 				return;
 			} else {
 				now = &(*now)->next;
@@ -880,22 +875,23 @@ static void main_print_stats(print_data_t *p_data)
 		}
 		if (*now == NULL) {
 			if (buffer->sys_call_number == 0) {
-				copy = copy_entry(buffer, free_list_head);
+				copy = create_hash_entry(buffer, flist_head);
 				list_add(copy, &hash_table[hash]);
-				return;
+			} else {
+				/* there is not call entry but only return entry */
+				add_no_pair_entry(buffer, stats);
 			}
-			/* there is not call entry but only return entry */
-			add_no_pair_entry(buffer, stats);
 		}
 	}
 }
 
 static void print_call_entry(const buffer_t *buf, FILE *w_fp)
 {
+	int nr_syscalls = kstrax_get_nr_syscalls();
 	long call_utime;
 	struct tm call_time;
 	
-	if (NR_syscalls < abs(buf->sys_call_number)) {
+	if (nr_syscalls < abs(buf->sys_call_number)) {
 		fprintf(w_fp, "Unknown syscall number : %d\n", 
 			buf->sys_call_number);
 		return;
@@ -917,10 +913,11 @@ static void print_call_entry(const buffer_t *buf, FILE *w_fp)
 
 static void print_return_entry(const buffer_t *buf, FILE *w_fp)
 {
+	int nr_syscalls = kstrax_get_nr_syscalls();
 	long ret_utime;
 	struct tm ret_time;
 
-	if (NR_syscalls < abs(buf->sys_call_number)) {
+	if (nr_syscalls < abs(buf->sys_call_number)) {
 		fprintf(w_fp, "Unknown syscall number : %d\n", -buf->sys_call_number);
 		return;
 	}
@@ -945,10 +942,11 @@ static void print_return_entry(const buffer_t *buf, FILE *w_fp)
 
 static void print_pair_entry(const buffer_t *call, const buffer_t *ret, FILE *w_fp)
 {
+	int nr_syscalls = kstrax_get_nr_syscalls();
 	long call_utime, ret_utime;
 	struct tm call_time, ret_time;
 
-	if (NR_syscalls < abs(call->sys_call_number)) {
+	if (nr_syscalls < abs(call->sys_call_number)) {
 		fprintf(w_fp, "Unknown syscall number : %d\n", call->sys_call_number);
 		return;
 	}
@@ -1042,35 +1040,13 @@ static void flush_hash_table(print_data_t *);
 static void stats_flush_hash_table(print_data_t *);
 static void stats_print(print_data_t *);
 static void stats_print_graph(print_data_t *);
-static void demo_print(print_data_t *); /* ken-patu */
 
 /* "-t" */
 static void post_print_normal(print_data_t *p_data)
 {
-	if (kstrax_gflag == 1) {
-		int i;
-		fprintf(p_data->w_fp, "count ");
-		for (i = 0; i < NR_syscalls; i++) {
-			fprintf(p_data->w_fp, "%ld ", kstrax_sum_count[i]);
-		}
-		fprintf(p_data->w_fp, "\ntotal ");
-		for (i = 0; i < NR_syscalls; i++) {
-			fprintf(p_data->w_fp, "%ld ", kstrax_sum_proc_time[i]);
-		}
-		fprintf(p_data->w_fp, "\naverage ");
-		for (i = 0; i < NR_syscalls; i++) {
-			if (kstrax_sum_count[i] == 0)
-				fprintf(p_data->w_fp, "0 ");
-			else
-				fprintf(p_data->w_fp, "%ld ", 
-					kstrax_sum_proc_time[i]/kstrax_sum_count[i]);
-		}
-
-	}
-
 	flush_hash_table(p_data);
-	list_free(&p_data->free_list);
 	free(p_data->hash_table);
+	list_free(&p_data->free_list);
 }
 
 /* "-r" */
@@ -1083,16 +1059,13 @@ static void post_print_raw(print_data_t *p_data)
 static void post_print_stats(print_data_t *p_data)
 {
 	stats_flush_hash_table(p_data);
-	if (kstrax_gflag == 0)
-		stats_print(p_data);
-	else
+	if (kstrax_gflag == 1) {
 		stats_print_graph(p_data);
-//#if 0
-	if (kstrax_dflag == 1)
-		demo_print(p_data);
-//#endif
+	} else {
+		stats_print(p_data);
+	}
 
-	free(p_data->stats);
+	free(p_data->stats_array);
 	list_free(&p_data->free_list);
 	free(p_data->hash_table);
 }
@@ -1100,12 +1073,13 @@ static void post_print_stats(print_data_t *p_data)
 static void flush_hash_table(print_data_t *p_data)
 {
 	int i;
+	int nr_syscalls = kstrax_get_nr_syscalls();
 
 	while (1) {
 		int count = 0;
 		kst_list_entry_t **tmp, **min = NULL;
 
-		for (i = 0; i < NR_syscalls; i++) {
+		for (i = 0; i < nr_syscalls; i++) {
 			tmp = &p_data->hash_table[i];
 			if (*tmp == NULL) {
 				count++;
@@ -1123,7 +1097,7 @@ static void flush_hash_table(print_data_t *p_data)
 		}
 		
 		/* hash_table is empty */
-		if (count == NR_syscalls) break;
+		if (count == nr_syscalls) break;
 
 		if (kstrax_gflag == 0)
 			print_call_entry(&(*min)->buf, p_data->w_fp);
@@ -1134,47 +1108,30 @@ static void flush_hash_table(print_data_t *p_data)
 
 static void stats_flush_hash_table(print_data_t *p_data)
 {
-	int i;
+	int index = 0;
+	int nr_syscalls = kstrax_get_nr_syscalls();
+	kst_list_entry_t **tmp;
+	kst_list_entry_t *del;
 
-	while (1) {
-		int count = 0;
-		kst_list_entry_t **tmp, **min = NULL;
-
-		for (i = 0; i < NR_syscalls; i++) {
-			tmp = &p_data->hash_table[i];
-			
-			if (*tmp == NULL) {
-				count++;
-				continue;
-			}
-
-			while ((*tmp)->next != NULL) {
-				tmp = &(*tmp)->next;
-			}
-
-			if (min == NULL || 
-			    (*min)->buf.serial > (*tmp)->buf.serial) {
-				min = tmp;
-			} 
+	while (index < nr_syscalls) {
+		tmp = &p_data->hash_table[index];
+		if (*tmp == NULL) {
+			index++;
+			continue;
 		}
-		
-		/* hash_table is empty */
-		if (count == NR_syscalls) break;
-
-		add_no_pair_entry(&(*min)->buf, p_data->stats);
-		list_add(*min, &p_data->free_list);
-		*min = NULL;
+		add_no_pair_entry(&(*tmp)->buf, p_data->stats_array);
+		del = list_del(tmp);		
+		list_add(del, &p_data->free_list);
 	}
 }
 
 static void stats_print(print_data_t *p_data)
 {
 	int i, nr_stats;
-	statistics_t *stats = p_data->stats;
-	unsigned long success;
-	proc_time_t average;
+	statistics_t *stats = p_data->stats_array;
+	unsigned long nr_pair;
 	
-	nr_stats = __get_nr_stats();
+	nr_stats = __get_nr_syscalls();
 
 	for (i = 0; i < nr_stats; i++) {
 		if (stats[i].count == 0)
@@ -1183,18 +1140,16 @@ static void stats_print(print_data_t *p_data)
 		__print_syscall_name_stats(p_data->w_fp, i);
 		fprintf(p_data->w_fp, "%7lu  ", stats[i].count);
 
-		if (stats[i].count  == 
-		    stats[i].nr_no_entry + stats[i].nr_no_return) {
+		nr_pair = stats[i].count - stats[i].nr_no_entry -
+			stats[i].nr_no_return;
+
+		if (nr_pair == 0) {
 			int j;
 			for (j = 0; j < 4; j++) 
 				fprintf(p_data->w_fp, "------.------  ");
 		} else {
-			success = stats[i].count - 
-				stats[i].nr_no_entry -
-				stats[i].nr_no_return;
-			average = stats[i].total_time / success;
-
-			__print_proc_time(p_data->w_fp, average);
+			__print_proc_time(p_data->w_fp, 
+					  (stats[i].total_time / nr_pair));
 			__print_proc_time(p_data->w_fp, stats[i].max_time);
 			__print_proc_time(p_data->w_fp, stats[i].min_time);
 			__print_proc_time(p_data->w_fp, stats[i].total_time);
@@ -1209,28 +1164,25 @@ static void stats_print(print_data_t *p_data)
 static void stats_print_graph(print_data_t *p_data)
 {
 	int i, nr_stats;
-	statistics_t *stats = p_data->stats;
-	unsigned long success;
-	proc_time_t average;
-	
-	nr_stats = __get_nr_stats();
+	statistics_t *stats = p_data->stats_array;
+	unsigned long nr_pair;
+
+	nr_stats = __get_nr_syscalls();
 	
 	for (i = 0; i < nr_stats; i++) {
 		__print_syscall_name_stats_graph(p_data->w_fp, i);
 		fprintf(p_data->w_fp, "%lu ", stats[i].count);
+
+		nr_pair = stats[i].count - stats[i].nr_no_entry -
+			stats[i].nr_no_return;
 		
-		if (stats[i].count  == 
-		    stats[i].nr_no_entry + stats[i].nr_no_return) {
+		if (nr_pair == 0) {
 			int j;
 			for (j = 0; j < 4; j++) 
 				fprintf(p_data->w_fp, "0.000000 ");
 		} else {
-			success = stats[i].count - 
-				stats[i].nr_no_entry -
-				stats[i].nr_no_return;
-			average = stats[i].total_time / success;
-			
-			__print_proc_time_graph(p_data->w_fp, average);
+			__print_proc_time_graph(p_data->w_fp,
+						(stats[i].total_time / nr_pair));
 			__print_proc_time_graph(p_data->w_fp, stats[i].max_time);
 			__print_proc_time_graph(p_data->w_fp, stats[i].min_time);
 			__print_proc_time_graph(p_data->w_fp, stats[i].total_time);
@@ -1242,68 +1194,35 @@ static void stats_print_graph(print_data_t *p_data)
 	}
 }
 
-/* ken-patu */
-static void demo_print(print_data_t *p_data)
-{
-	int i, j, k = 0;
-	statistics_t *stats = p_data->stats;
-	unsigned long success;
-	long long average;
-	
-	for (i = 0; i < NR_syscalls; i++) {
-		if (stats[i].count == 0)
-			continue;
-
-		if (i != KS_READ && i != KS_WRITE && i != KS_OPEN && 
-		    i != KS_CLOSE && i != KS_EXECVE &&// i != KS_TIME && 
-		    i != KS_GETTIMEOFDAY && i != KS_CLONE)
-			continue;
-   
-		fprintf(p_data->w_fp, "%d ", ++k);
-
-		if (stats[i].count  == 
-		    stats[i].nr_no_entry + stats[i].nr_no_return) {
-			for (j = 0; j < 4; j++) 
-				fprintf(p_data->w_fp, "------.------  ");
-		} else {
-			success = stats[i].count - 
-				stats[i].nr_no_entry -
-				stats[i].nr_no_return;
-			average = stats[i].total_time / success;
-//			fprintf(p_data->w_fp, "%lld %lld\n", stats[i].total_time, average);			
-			fprintf(p_data->w_fp, "%lld\n", average);
-		}		
-	}
-}
 /*-------------------------------------------------------------------------------
  *
  */
-static kst_list_entry_t *copy_entry(const buffer_t *buf, 
-				    kst_list_entry_t **free_list_head)
+static kst_list_entry_t *create_hash_entry(const buffer_t *buf, 
+					   kst_list_entry_t **flist_head)
 {
 	kst_list_entry_t *ret;
 	
 	/* create cache */
-	if (*free_list_head == NULL) {
+	if (*flist_head == NULL) {
 		int i;
 		for (i = 0; i < 50; i++) { /* create 50 entry for one time */
 			kst_list_entry_t *tmp;
 			tmp = (kst_list_entry_t *)malloc(sizeof(kst_list_entry_t));
 			if (tmp == NULL) {
-				perror("copy_entry(malloc)");
+				perror("create_hash_entry(malloc)");
 				exit(1);
 			}
 			if (i == 0) {
 				tmp->next = NULL;
 			} else {
-				tmp->next = *free_list_head;
+				tmp->next = *flist_head;
 			}
-			*free_list_head = tmp;
+			*flist_head = tmp;
 		}
 	}
 
-	ret = *free_list_head;
-	*free_list_head = ret->next;
+	ret = *flist_head;
+	*flist_head = ret->next;
 	ret->buf = *buf;
 	ret->next = NULL;
 	return ret;
@@ -1336,11 +1255,11 @@ static void add_no_pair_entry(const buffer_t *buf, statistics_t *stats)
 	int number;
 	number = __get_stats_array_index(buf);
 
-	if (number > 0)
+	if (number > 0) {
 		stats[number].nr_no_return++;
-	else
+	} else {
 		stats[-number].nr_no_entry++;
-
+	}
 	stats[abs(number)].count++;
 }
 
